#include "m_config.h"
#include <stdlib.h> /* EXIT_SUCCESS, EXIT_FAILURE, srand, rand */
#include <check.h>

#include <mstdlib/mstdlib.h>

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

#define EST5EDT "EST5EDT,M3.2.0/02:00:00,M11.1.0/02:00:00"
#define PST8PDT "PST8PDT,M3.2.0/02:00:00,M11.1.0/02:00:00"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

extern Suite *M_time_fmt_suite(void);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static struct {
	M_time_t ts;
	const char *zone;
	const char *tostr_fmt;
	const char *abbr;
	const char *s;
} check_fmts[] = {
	/* STD */
	{ 1361530800, "EST5EDT", "%Y/%m/%d %H",             "EST", "2013/02/22 06"                   },
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S%z",     "EST", "2013/02/22 18:59:13-0500"        },
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S %z",    "EST", "2013/02/22 18:59:13 -0500"       },
#if 0
	/* XXX: Seconds are not requires for %z but to string always puts them so the check for parsing
 	 * without minutes won't compare right to the output. */
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S%z",     "EST", "2013/02/22 18:59:13-05"          },
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S %z",    "EST", "2013/02/22 18:59:13 -05"         },
#endif
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S",       "EST", "2013/02/22 18:59:13"             },
	{ 1361577553, "PST8PDT", "%Y/%m/%d %H:%M:%S%z",     "PST", "2013/02/22 15:59:13-0800"        },
	{ 1361577553, "PST8PDT", "%Y/%m/%d %H:%M:%S %z",    "PST", "2013/02/22 15:59:13 -0800"       },
	{ 1361577553, "PST8PDT", "%Y/%m/%d %H:%M:%S",       "PST", "2013/02/22 15:59:13"             },

	{ 1361577553, "PST8PDT", "%Y/%m/%d %H:%M:%S",       "PST", "2013/02/22 15:59:13"             },

	{ 1361577553, "EST5EDT", "%Y/%m/%d T %H:%M:%S%z",   "EST", "2013/02/22 T 18:59:13-0500"      },
	{ 1361577553, "EST5EDT", "%Y/%m/%d T %H:%M:%S %z",  "EST", "2013/02/22 T 18:59:13 -0500"     },
	{ 1361577553, "EST5EDT", "%Y/%m/%d T %H:%M:%S",     "EST", "2013/02/22 T 18:59:13"           },
	{ 1361577553, "PST8PDT", "%Y/%m/%d T %H:%M:%S%z",   "PST", "2013/02/22 T 15:59:13-0800"      },
	{ 1361577553, "PST8PDT", "%Y/%m/%d T %H:%M:%S %z",  "PST", "2013/02/22 T 15:59:13 -0800"     },
	{ 1361577553, "PST8PDT", "%Y/%m/%d T %H:%M:%S",     "PST", "2013/02/22 T 15:59:13"           },

	{ 1361530800, "EST5EDT", "%Y/%m/%d %H%p",           "EST", "2013/02/22 06am"               },
	{ 1361577553, "EST5EDT", "%Y/%m/%d T %H:%M:%S %P",  "EST", "2013/02/22 T 18:59:13 PM"        },
	{ 1361512753, "EST5EDT", "%Y/%m/%dT%H:%M:%S %P",    "EST", "2013/02/22T00:59:13 AM"          },
	{ 1361555953, "EST5EDT", "%Y/%m/%d T%H:%M:%S %p",   "EST", "2013/02/22 T12:59:13 pm"         },

	/* DST */
	{ 1377212353, "EST5EDT", "%Y/%m/%d %H:%M:%S%z",     "EDT", "2013/08/22 18:59:13-0400"        },
	{ 1377212353, "EST5EDT", "%Y/%m/%d %H:%M:%S %z",    "EDT", "2013/08/22 18:59:13 -0400"       },
	{ 1377212353, "EST5EDT", "%Y/%m/%d %H:%M:%S",       "EDT", "2013/08/22 18:59:13"             },
	{ 1377212353, "PST8PDT", "%Y/%m/%d %H:%M:%S%z",     "PDT", "2013/08/22 15:59:13-0700"        },
	{ 1377212353, "PST8PDT", "%Y/%m/%d %H:%M:%S %z",    "PDT", "2013/08/22 15:59:13 -0700"       },
	{ 1377212353, "PST8PDT", "%Y/%m/%d %H:%M:%S",       "PDT", "2013/08/22 15:59:13"             },

	{ 1377212353, "EST5EDT", "%Y/%m/%dT%H:%M:%S%z",     "EDT", "2013/08/22T18:59:13-0400"        },
	{ 1377212353, "EST5EDT", "%Y/%m/%dT%H:%M:%S %z",    "EDT", "2013/08/22T18:59:13 -0400"       },
	{ 1377212353, "EST5EDT", "%Y/%m/%dT%H:%M:%S",       "EDT", "2013/08/22T18:59:13"             },
	{ 1377212353, "PST8PDT", "%Y/%m/%dT%H:%M:%S%z",     "PDT", "2013/08/22T15:59:13-0700"        },
	{ 1377212353, "PST8PDT", "%Y/%m/%dT%H:%M:%S %z",    "PDT", "2013/08/22T15:59:13 -0700"       },
	{ 1377212353, "PST8PDT", "%Y/%m/%dT%H:%M:%S",       "PDT", "2013/08/22T15:59:13"             },

	/* Every single format... */
	{ 1467651600, "EST5EDT", "%m/%d/%Y %H",             "EDT", "07/04/2016 13"                   },
	{ 1467651600, "EST5EDT", "%m/%d/%Y %H %P",          "EDT", "07/04/2016 13 PM"                },
	{ 1467651600, "EST5EDT", "%m/%d/%Y %H %p",          "EDT", "07/04/2016 13 pm"                },
	{ 1467651600, "EST5EDT", "%m/%d/%Y %I %P",          "EDT", "07/04/2016 01 PM"                },
	{ 1467651600, "EST5EDT", "%m/%d/%Y %I %p",          "EDT", "07/04/2016 01 pm"                },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H%M",           "EDT", "07/04/2016 1302"                 },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H%M %P",        "EDT", "07/04/2016 1302 PM"              },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H%M %p",        "EDT", "07/04/2016 1302 pm"              },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %I%M %P",        "EDT", "07/04/2016 0102 PM"              },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %I%M %p",        "EDT", "07/04/2016 0102 pm"              },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H%M%S",         "EDT", "07/04/2016 130242"               },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H%M%S %P",      "EDT", "07/04/2016 130242 PM"            },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H%M%S %p",      "EDT", "07/04/2016 130242 pm"            },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %I%M%S %P",      "EDT", "07/04/2016 010242 PM"            },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %I%M%S %p",      "EDT", "07/04/2016 010242 pm"            },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H%M%S %z",      "EDT", "07/04/2016 130242 -0400"         },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H:%M",          "EDT", "07/04/2016 13:02"                },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H:%M %P",       "EDT", "07/04/2016 13:02 PM"             },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H:%M %p",       "EDT", "07/04/2016 13:02 pm"             },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %I:%M %P",       "EDT", "07/04/2016 01:02 PM"             },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %I:%M %p",       "EDT", "07/04/2016 01:02 pm"             },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H:%M:%S",       "EDT", "07/04/2016 13:02:42"             },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H:%M:%S %P",    "EDT", "07/04/2016 13:02:42 PM"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H:%M:%S %p",    "EDT", "07/04/2016 13:02:42 pm"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %I:%M:%S %P",    "EDT", "07/04/2016 01:02:42 PM"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %I:%M:%S %p",    "EDT", "07/04/2016 01:02:42 pm"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H:%M:%S %z",    "EDT", "07/04/2016 13:02:42 -0400"       },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H-%M",          "EDT", "07/04/2016 13-02"                },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H-%M %P",       "EDT", "07/04/2016 13-02 PM"             },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %H-%M %p",       "EDT", "07/04/2016 13-02 pm"             },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %I-%M %P",       "EDT", "07/04/2016 01-02 PM"             },
	{ 1467651720, "EST5EDT", "%m/%d/%Y %I-%M %p",       "EDT", "07/04/2016 01-02 pm"             },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H-%M-%S",       "EDT", "07/04/2016 13-02-42"             },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H-%M-%S %P",    "EDT", "07/04/2016 13-02-42 PM"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H-%M-%S %p",    "EDT", "07/04/2016 13-02-42 pm"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %I-%M-%S %P",    "EDT", "07/04/2016 01-02-42 PM"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %I-%M-%S %p",    "EDT", "07/04/2016 01-02-42 pm"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y %H-%M-%S %z",    "EDT", "07/04/2016 13-02-42 -0400"       },
                 
	{ 1467651600, "EST5EDT", "%m/%d/%Y T %H",           "EDT", "07/04/2016 T 13"                 },
	{ 1467651600, "EST5EDT", "%m/%d/%Y T %H %P",        "EDT", "07/04/2016 T 13 PM"              },
	{ 1467651600, "EST5EDT", "%m/%d/%Y T %H %p",        "EDT", "07/04/2016 T 13 pm"              },
	{ 1467651600, "EST5EDT", "%m/%d/%Y T %I %P",        "EDT", "07/04/2016 T 01 PM"              },
	{ 1467651600, "EST5EDT", "%m/%d/%Y T %I %p",        "EDT", "07/04/2016 T 01 pm"              },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H%M",         "EDT", "07/04/2016 T 1302"               },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H%M %P",      "EDT", "07/04/2016 T 1302 PM"            },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H%M %p",      "EDT", "07/04/2016 T 1302 pm"            },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %I%M %P",      "EDT", "07/04/2016 T 0102 PM"            },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %I%M %p",      "EDT", "07/04/2016 T 0102 pm"            },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H%M%S",       "EDT", "07/04/2016 T 130242"             },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H%M%S %P",    "EDT", "07/04/2016 T 130242 PM"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H%M%S %p",    "EDT", "07/04/2016 T 130242 pm"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %I%M%S %P",    "EDT", "07/04/2016 T 010242 PM"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %I%M%S %p",    "EDT", "07/04/2016 T 010242 pm"          },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H%M%S %z",    "EDT", "07/04/2016 T 130242 -0400"       },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H:%M",        "EDT", "07/04/2016 T 13:02"              },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H:%M %P",     "EDT", "07/04/2016 T 13:02 PM"           },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H:%M %p",     "EDT", "07/04/2016 T 13:02 pm"           },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %I:%M %P",     "EDT", "07/04/2016 T 01:02 PM"           },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %I:%M %p",     "EDT", "07/04/2016 T 01:02 pm"           },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H:%M:%S",     "EDT", "07/04/2016 T 13:02:42"           },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H:%M:%S %P",  "EDT", "07/04/2016 T 13:02:42 PM"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H:%M:%S %p",  "EDT", "07/04/2016 T 13:02:42 pm"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %I:%M:%S %P",  "EDT", "07/04/2016 T 01:02:42 PM"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %I:%M:%S %p",  "EDT", "07/04/2016 T 01:02:42 pm"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H:%M:%S %z",  "EDT", "07/04/2016 T 13:02:42 -0400"     },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H-%M",        "EDT", "07/04/2016 T 13-02"              },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H-%M %P",     "EDT", "07/04/2016 T 13-02 PM"           },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %H-%M %p",     "EDT", "07/04/2016 T 13-02 pm"           },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %I-%M %P",     "EDT", "07/04/2016 T 01-02 PM"           },
	{ 1467651720, "EST5EDT", "%m/%d/%Y T %I-%M %p",     "EDT", "07/04/2016 T 01-02 pm"           },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H-%M-%S",     "EDT", "07/04/2016 T 13-02-42"           },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H-%M-%S %P",  "EDT", "07/04/2016 T 13-02-42 PM"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H-%M-%S %p",  "EDT", "07/04/2016 T 13-02-42 pm"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %I-%M-%S %P",  "EDT", "07/04/2016 T 01-02-42 PM"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %I-%M-%S %p",  "EDT", "07/04/2016 T 01-02-42 pm"        },
	{ 1467651762, "EST5EDT", "%m/%d/%Y T %H-%M-%S %z",  "EDT", "07/04/2016 T 13-02-42 -0400"     },
                 
	{  1467651600, "EST5EDT", "%m-%d-%Y %H",            "EDT",  "07-04-2016 13"                  },
	{  1467651600, "EST5EDT", "%m-%d-%Y %H %P",         "EDT",  "07-04-2016 13 PM"               },
	{  1467651600, "EST5EDT", "%m-%d-%Y %H %p",         "EDT",  "07-04-2016 13 pm"               },
	{  1467651600, "EST5EDT", "%m-%d-%Y %I %P",         "EDT",  "07-04-2016 01 PM"               },
	{  1467651600, "EST5EDT", "%m-%d-%Y %I %p",         "EDT",  "07-04-2016 01 pm"               },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H%M",          "EDT",  "07-04-2016 1302"                },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H%M %P",       "EDT",  "07-04-2016 1302 PM"             },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H%M %p",       "EDT",  "07-04-2016 1302 pm"             },
	{  1467651720, "EST5EDT", "%m-%d-%Y %I%M %P",       "EDT",  "07-04-2016 0102 PM"             },
	{  1467651720, "EST5EDT", "%m-%d-%Y %I%M %p",       "EDT",  "07-04-2016 0102 pm"             },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H%M%S",        "EDT",  "07-04-2016 130242"              },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H%M%S %P",     "EDT",  "07-04-2016 130242 PM"           },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H%M%S %p",     "EDT",  "07-04-2016 130242 pm"           },
	{  1467651762, "EST5EDT", "%m-%d-%Y %I%M%S %P",     "EDT",  "07-04-2016 010242 PM"           },
	{  1467651762, "EST5EDT", "%m-%d-%Y %I%M%S %p",     "EDT",  "07-04-2016 010242 pm"           },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H%M%S %z",     "EDT",  "07-04-2016 130242 -0400"        },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H:%M",         "EDT",  "07-04-2016 13:02"               },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H:%M %P",      "EDT",  "07-04-2016 13:02 PM"            },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H:%M %p",      "EDT",  "07-04-2016 13:02 pm"            },
	{  1467651720, "EST5EDT", "%m-%d-%Y %I:%M %P",      "EDT",  "07-04-2016 01:02 PM"            },
	{  1467651720, "EST5EDT", "%m-%d-%Y %I:%M %p",      "EDT",  "07-04-2016 01:02 pm"            },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H:%M:%S",      "EDT",  "07-04-2016 13:02:42"            },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H:%M:%S %P",   "EDT",  "07-04-2016 13:02:42 PM"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H:%M:%S %p",   "EDT",  "07-04-2016 13:02:42 pm"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %I:%M:%S %P",   "EDT",  "07-04-2016 01:02:42 PM"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %I:%M:%S %p",   "EDT",  "07-04-2016 01:02:42 pm"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H:%M:%S %z",   "EDT",  "07-04-2016 13:02:42 -0400"      },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H-%M",         "EDT",  "07-04-2016 13-02"               },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H-%M %P",      "EDT",  "07-04-2016 13-02 PM"            },
	{  1467651720, "EST5EDT", "%m-%d-%Y %H-%M %p",      "EDT",  "07-04-2016 13-02 pm"            },
	{  1467651720, "EST5EDT", "%m-%d-%Y %I-%M %P",      "EDT",  "07-04-2016 01-02 PM"            },
	{  1467651720, "EST5EDT", "%m-%d-%Y %I-%M %p",      "EDT",  "07-04-2016 01-02 pm"            },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H-%M-%S",      "EDT",  "07-04-2016 13-02-42"            },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H-%M-%S %P",   "EDT",  "07-04-2016 13-02-42 PM"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H-%M-%S %p",   "EDT",  "07-04-2016 13-02-42 pm"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %I-%M-%S %P",   "EDT",  "07-04-2016 01-02-42 PM"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %I-%M-%S %p",   "EDT",  "07-04-2016 01-02-42 pm"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y %H-%M-%S %z",   "EDT",  "07-04-2016 13-02-42 -0400"      },

	{  1467651600, "EST5EDT", "%m-%d-%Y T %H",          "EDT",  "07-04-2016 T 13"                },
	{  1467651600, "EST5EDT", "%m-%d-%Y T %H %P",       "EDT",  "07-04-2016 T 13 PM"             },
	{  1467651600, "EST5EDT", "%m-%d-%Y T %H %p",       "EDT",  "07-04-2016 T 13 pm"             },
	{  1467651600, "EST5EDT", "%m-%d-%Y T %I %P",       "EDT",  "07-04-2016 T 01 PM"             },
	{  1467651600, "EST5EDT", "%m-%d-%Y T %I %p",       "EDT",  "07-04-2016 T 01 pm"             },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H%M",        "EDT",  "07-04-2016 T 1302"              },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H%M %P",     "EDT",  "07-04-2016 T 1302 PM"           },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H%M %p",     "EDT",  "07-04-2016 T 1302 pm"           },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %I%M %P",     "EDT",  "07-04-2016 T 0102 PM"           },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %I%M %p",     "EDT",  "07-04-2016 T 0102 pm"           },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H%M%S",      "EDT",  "07-04-2016 T 130242"            },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H%M%S %P",   "EDT",  "07-04-2016 T 130242 PM"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H%M%S %p",   "EDT",  "07-04-2016 T 130242 pm"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %I%M%S %P",   "EDT",  "07-04-2016 T 010242 PM"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %I%M%S %p",   "EDT",  "07-04-2016 T 010242 pm"         },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H%M%S %z",   "EDT",  "07-04-2016 T 130242 -0400"      },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H:%M",       "EDT",  "07-04-2016 T 13:02"             },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H:%M %P",    "EDT",  "07-04-2016 T 13:02 PM"          },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H:%M %p",    "EDT",  "07-04-2016 T 13:02 pm"          },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %I:%M %P",    "EDT",  "07-04-2016 T 01:02 PM"          },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %I:%M %p",    "EDT",  "07-04-2016 T 01:02 pm"          },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H:%M:%S",    "EDT",  "07-04-2016 T 13:02:42"          },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H:%M:%S %P", "EDT",  "07-04-2016 T 13:02:42 PM"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H:%M:%S %p", "EDT",  "07-04-2016 T 13:02:42 pm"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %I:%M:%S %P", "EDT",  "07-04-2016 T 01:02:42 PM"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %I:%M:%S %p", "EDT",  "07-04-2016 T 01:02:42 pm"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H:%M:%S %z", "EDT",  "07-04-2016 T 13:02:42 -0400"    },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H-%M",       "EDT",  "07-04-2016 T 13-02"             },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H-%M %P",    "EDT",  "07-04-2016 T 13-02 PM"          },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %H-%M %p",    "EDT",  "07-04-2016 T 13-02 pm"          },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %I-%M %P",    "EDT",  "07-04-2016 T 01-02 PM"          },
	{  1467651720, "EST5EDT", "%m-%d-%Y T %I-%M %p",    "EDT",  "07-04-2016 T 01-02 pm"          },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H-%M-%S",    "EDT",  "07-04-2016 T 13-02-42"          },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H-%M-%S %P", "EDT",  "07-04-2016 T 13-02-42 PM"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H-%M-%S %p", "EDT",  "07-04-2016 T 13-02-42 pm"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %I-%M-%S %P", "EDT",  "07-04-2016 T 01-02-42 PM"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %I-%M-%S %p", "EDT",  "07-04-2016 T 01-02-42 pm"       },
	{  1467651762, "EST5EDT", "%m-%d-%Y T %H-%M-%S %z", "EDT",  "07-04-2016 T 13-02-42 -0400"    },

	{ 1467651600, "EST5EDT", "%m/%d/%y %H",             "EDT",  "07/04/16 13"                  },
	{ 1467651600, "EST5EDT", "%m/%d/%y %H %P",          "EDT",  "07/04/16 13 PM"               },
	{ 1467651600, "EST5EDT", "%m/%d/%y %H %p",          "EDT",  "07/04/16 13 pm"               },
	{ 1467651600, "EST5EDT", "%m/%d/%y %I %P",          "EDT",  "07/04/16 01 PM"               },
	{ 1467651600, "EST5EDT", "%m/%d/%y %I %p",          "EDT",  "07/04/16 01 pm"               },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H%M",           "EDT",  "07/04/16 1302"                },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H%M %P",        "EDT",  "07/04/16 1302 PM"             },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H%M %p",        "EDT",  "07/04/16 1302 pm"             },
	{ 1467651720, "EST5EDT", "%m/%d/%y %I%M %P",        "EDT",  "07/04/16 0102 PM"             },
	{ 1467651720, "EST5EDT", "%m/%d/%y %I%M %p",        "EDT",  "07/04/16 0102 pm"             },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H%M%S",         "EDT",  "07/04/16 130242"              },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H%M%S %P",      "EDT",  "07/04/16 130242 PM"           },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H%M%S %p",      "EDT",  "07/04/16 130242 pm"           },
	{ 1467651762, "EST5EDT", "%m/%d/%y %I%M%S %P",      "EDT",  "07/04/16 010242 PM"           },
	{ 1467651762, "EST5EDT", "%m/%d/%y %I%M%S %p",      "EDT",  "07/04/16 010242 pm"           },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H%M%S %z",      "EDT",  "07/04/16 130242 -0400"        },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H:%M",          "EDT",  "07/04/16 13:02"               },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H:%M %P",       "EDT",  "07/04/16 13:02 PM"            },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H:%M %p",       "EDT",  "07/04/16 13:02 pm"            },
	{ 1467651720, "EST5EDT", "%m/%d/%y %I:%M %P",       "EDT",  "07/04/16 01:02 PM"            },
	{ 1467651720, "EST5EDT", "%m/%d/%y %I:%M %p",       "EDT",  "07/04/16 01:02 pm"            },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H:%M:%S",       "EDT",  "07/04/16 13:02:42"            },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H:%M:%S %P",    "EDT",  "07/04/16 13:02:42 PM"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H:%M:%S %p",    "EDT",  "07/04/16 13:02:42 pm"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %I:%M:%S %P",    "EDT",  "07/04/16 01:02:42 PM"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %I:%M:%S %p",    "EDT",  "07/04/16 01:02:42 pm"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H:%M:%S %z",    "EDT",  "07/04/16 13:02:42 -0400"      },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H-%M",          "EDT",  "07/04/16 13-02"               },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H-%M %P",       "EDT",  "07/04/16 13-02 PM"            },
	{ 1467651720, "EST5EDT", "%m/%d/%y %H-%M %p",       "EDT",  "07/04/16 13-02 pm"            },
	{ 1467651720, "EST5EDT", "%m/%d/%y %I-%M %P",       "EDT",  "07/04/16 01-02 PM"            },
	{ 1467651720, "EST5EDT", "%m/%d/%y %I-%M %p",       "EDT",  "07/04/16 01-02 pm"            },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H-%M-%S",       "EDT",  "07/04/16 13-02-42"            },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H-%M-%S %P",    "EDT",  "07/04/16 13-02-42 PM"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H-%M-%S %p",    "EDT",  "07/04/16 13-02-42 pm"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %I-%M-%S %P",    "EDT",  "07/04/16 01-02-42 PM"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %I-%M-%S %p",    "EDT",  "07/04/16 01-02-42 pm"         },
	{ 1467651762, "EST5EDT", "%m/%d/%y %H-%M-%S %z",    "EDT",  "07/04/16 13-02-42 -0400"      },

	{ 1467651600, "EST5EDT", "%m/%d/%y T %H",           "EDT",  "07/04/16 T 13"                }, 
	{ 1467651600, "EST5EDT", "%m/%d/%y T %H %P",        "EDT",  "07/04/16 T 13 PM"             }, 
	{ 1467651600, "EST5EDT", "%m/%d/%y T %H %p",        "EDT",  "07/04/16 T 13 pm"             }, 
	{ 1467651600, "EST5EDT", "%m/%d/%y T %I %P",        "EDT",  "07/04/16 T 01 PM"             }, 
	{ 1467651600, "EST5EDT", "%m/%d/%y T %I %p",        "EDT",  "07/04/16 T 01 pm"             }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H%M",         "EDT",  "07/04/16 T 1302"              }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H%M %P",      "EDT",  "07/04/16 T 1302 PM"           }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H%M %p",      "EDT",  "07/04/16 T 1302 pm"           }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %I%M %P",      "EDT",  "07/04/16 T 0102 PM"           }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %I%M %p",      "EDT",  "07/04/16 T 0102 pm"           }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H%M%S",       "EDT",  "07/04/16 T 130242"            }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H%M%S %P",    "EDT",  "07/04/16 T 130242 PM"         }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H%M%S %p",    "EDT",  "07/04/16 T 130242 pm"         }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %I%M%S %P",    "EDT",  "07/04/16 T 010242 PM"         }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %I%M%S %p",    "EDT",  "07/04/16 T 010242 pm"         }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H%M%S %z",    "EDT",  "07/04/16 T 130242 -0400"      }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H:%M",        "EDT",  "07/04/16 T 13:02"             }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H:%M %P",     "EDT",  "07/04/16 T 13:02 PM"          }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H:%M %p",     "EDT",  "07/04/16 T 13:02 pm"          }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %I:%M %P",     "EDT",  "07/04/16 T 01:02 PM"          }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %I:%M %p",     "EDT",  "07/04/16 T 01:02 pm"          }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H:%M:%S",     "EDT",  "07/04/16 T 13:02:42"          }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H:%M:%S %P",  "EDT",  "07/04/16 T 13:02:42 PM"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H:%M:%S %p",  "EDT",  "07/04/16 T 13:02:42 pm"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %I:%M:%S %P",  "EDT",  "07/04/16 T 01:02:42 PM"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %I:%M:%S %p",  "EDT",  "07/04/16 T 01:02:42 pm"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H:%M:%S %z",  "EDT",  "07/04/16 T 13:02:42 -0400"    }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H-%M",        "EDT",  "07/04/16 T 13-02"             }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H-%M %P",     "EDT",  "07/04/16 T 13-02 PM"          }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %H-%M %p",     "EDT",  "07/04/16 T 13-02 pm"          }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %I-%M %P",     "EDT",  "07/04/16 T 01-02 PM"          }, 
	{ 1467651720, "EST5EDT", "%m/%d/%y T %I-%M %p",     "EDT",  "07/04/16 T 01-02 pm"          }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H-%M-%S",     "EDT",  "07/04/16 T 13-02-42"          }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H-%M-%S %P",  "EDT",  "07/04/16 T 13-02-42 PM"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H-%M-%S %p",  "EDT",  "07/04/16 T 13-02-42 pm"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %I-%M-%S %P",  "EDT",  "07/04/16 T 01-02-42 PM"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %I-%M-%S %p",  "EDT",  "07/04/16 T 01-02-42 pm"       }, 
	{ 1467651762, "EST5EDT", "%m/%d/%y T %H-%M-%S %z",  "EDT",  "07/04/16 T 13-02-42 -0400"    }, 
                 
	{ 1467651600, "EST5EDT", "%Y/%m/%d %H",             "EDT",  "2016/07/04 13"                  },
	{ 1467651600, "EST5EDT", "%Y/%m/%d %H %P",          "EDT",  "2016/07/04 13 PM"               },
	{ 1467651600, "EST5EDT", "%Y/%m/%d %H %p",          "EDT",  "2016/07/04 13 pm"               },
	{ 1467651600, "EST5EDT", "%Y/%m/%d %I %P",          "EDT",  "2016/07/04 01 PM"               },
	{ 1467651600, "EST5EDT", "%Y/%m/%d %I %p",          "EDT",  "2016/07/04 01 pm"               },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H%M",           "EDT",  "2016/07/04 1302"                },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H%M %P",        "EDT",  "2016/07/04 1302 PM"             },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H%M %p",        "EDT",  "2016/07/04 1302 pm"             },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %I%M %P",        "EDT",  "2016/07/04 0102 PM"             },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %I%M %p",        "EDT",  "2016/07/04 0102 pm"             },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H%M%S",         "EDT",  "2016/07/04 130242"              },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H%M%S %P",      "EDT",  "2016/07/04 130242 PM"           },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H%M%S %p",      "EDT",  "2016/07/04 130242 pm"           },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %I%M%S %P",      "EDT",  "2016/07/04 010242 PM"           },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %I%M%S %p",      "EDT",  "2016/07/04 010242 pm"           },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H%M%S %z",      "EDT",  "2016/07/04 130242 -0400"        },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H:%M",          "EDT",  "2016/07/04 13:02"               },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H:%M %P",       "EDT",  "2016/07/04 13:02 PM"            },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H:%M %p",       "EDT",  "2016/07/04 13:02 pm"            },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %I:%M %P",       "EDT",  "2016/07/04 01:02 PM"            },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %I:%M %p",       "EDT",  "2016/07/04 01:02 pm"            },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H:%M:%S",       "EDT",  "2016/07/04 13:02:42"            },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H:%M:%S %P",    "EDT",  "2016/07/04 13:02:42 PM"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H:%M:%S %p",    "EDT",  "2016/07/04 13:02:42 pm"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %I:%M:%S %P",    "EDT",  "2016/07/04 01:02:42 PM"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %I:%M:%S %p",    "EDT",  "2016/07/04 01:02:42 pm"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H:%M:%S %z",    "EDT",  "2016/07/04 13:02:42 -0400"      },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H-%M",          "EDT",  "2016/07/04 13-02"               },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H-%M %P",       "EDT",  "2016/07/04 13-02 PM"            },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %H-%M %p",       "EDT",  "2016/07/04 13-02 pm"            },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %I-%M %P",       "EDT",  "2016/07/04 01-02 PM"            },
	{ 1467651720, "EST5EDT", "%Y/%m/%d %I-%M %p",       "EDT",  "2016/07/04 01-02 pm"            },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H-%M-%S",       "EDT",  "2016/07/04 13-02-42"            },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H-%M-%S %P",    "EDT",  "2016/07/04 13-02-42 PM"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H-%M-%S %p",    "EDT",  "2016/07/04 13-02-42 pm"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %I-%M-%S %P",    "EDT",  "2016/07/04 01-02-42 PM"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %I-%M-%S %p",    "EDT",  "2016/07/04 01-02-42 pm"         },
	{ 1467651762, "EST5EDT", "%Y/%m/%d %H-%M-%S %z",    "EDT",  "2016/07/04 13-02-42 -0400"      },
                                                                                                 
	{ 1467651600, "EST5EDT", "%Y/%m/%d T %H",           "EDT",  "2016/07/04 T 13"                }, 
	{ 1467651600, "EST5EDT", "%Y/%m/%d T %H %P",        "EDT",  "2016/07/04 T 13 PM"             }, 
	{ 1467651600, "EST5EDT", "%Y/%m/%d T %H %p",        "EDT",  "2016/07/04 T 13 pm"             }, 
	{ 1467651600, "EST5EDT", "%Y/%m/%d T %I %P",        "EDT",  "2016/07/04 T 01 PM"             }, 
	{ 1467651600, "EST5EDT", "%Y/%m/%d T %I %p",        "EDT",  "2016/07/04 T 01 pm"             }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H%M",         "EDT",  "2016/07/04 T 1302"              }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H%M %P",      "EDT",  "2016/07/04 T 1302 PM"           }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H%M %p",      "EDT",  "2016/07/04 T 1302 pm"           }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %I%M %P",      "EDT",  "2016/07/04 T 0102 PM"           }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %I%M %p",      "EDT",  "2016/07/04 T 0102 pm"           }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H%M%S",       "EDT",  "2016/07/04 T 130242"            }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H%M%S %P",    "EDT",  "2016/07/04 T 130242 PM"         }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H%M%S %p",    "EDT",  "2016/07/04 T 130242 pm"         }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %I%M%S %P",    "EDT",  "2016/07/04 T 010242 PM"         }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %I%M%S %p",    "EDT",  "2016/07/04 T 010242 pm"         }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H%M%S %z",    "EDT",  "2016/07/04 T 130242 -0400"      }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H:%M",        "EDT",  "2016/07/04 T 13:02"             }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H:%M %P",     "EDT",  "2016/07/04 T 13:02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H:%M %p",     "EDT",  "2016/07/04 T 13:02 pm"          }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %I:%M %P",     "EDT",  "2016/07/04 T 01:02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %I:%M %p",     "EDT",  "2016/07/04 T 01:02 pm"          }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H:%M:%S",     "EDT",  "2016/07/04 T 13:02:42"          }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H:%M:%S %P",  "EDT",  "2016/07/04 T 13:02:42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H:%M:%S %p",  "EDT",  "2016/07/04 T 13:02:42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %I:%M:%S %P",  "EDT",  "2016/07/04 T 01:02:42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %I:%M:%S %p",  "EDT",  "2016/07/04 T 01:02:42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H:%M:%S %z",  "EDT",  "2016/07/04 T 13:02:42 -0400"    }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H-%M",        "EDT",  "2016/07/04 T 13-02"             }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H-%M %P",     "EDT",  "2016/07/04 T 13-02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %H-%M %p",     "EDT",  "2016/07/04 T 13-02 pm"          }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %I-%M %P",     "EDT",  "2016/07/04 T 01-02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y/%m/%d T %I-%M %p",     "EDT",  "2016/07/04 T 01-02 pm"          }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H-%M-%S",     "EDT",  "2016/07/04 T 13-02-42"          }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H-%M-%S %P",  "EDT",  "2016/07/04 T 13-02-42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H-%M-%S %p",  "EDT",  "2016/07/04 T 13-02-42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %I-%M-%S %P",  "EDT",  "2016/07/04 T 01-02-42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %I-%M-%S %p",  "EDT",  "2016/07/04 T 01-02-42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y/%m/%d T %H-%M-%S %z",  "EDT",  "2016/07/04 T 13-02-42 -0400"    }, 

	{ 1467651600, "EST5EDT", "%Y-%m-%d %H",             "EDT",  "2016-07-04 13"                  },
	{ 1467651600, "EST5EDT", "%Y-%m-%d %H %P",          "EDT",  "2016-07-04 13 PM"               },
	{ 1467651600, "EST5EDT", "%Y-%m-%d %H %p",          "EDT",  "2016-07-04 13 pm"               },
	{ 1467651600, "EST5EDT", "%Y-%m-%d %I %P",          "EDT",  "2016-07-04 01 PM"               },
	{ 1467651600, "EST5EDT", "%Y-%m-%d %I %p",          "EDT",  "2016-07-04 01 pm"               },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H%M",           "EDT",  "2016-07-04 1302"                },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H%M %P",        "EDT",  "2016-07-04 1302 PM"             },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H%M %p",        "EDT",  "2016-07-04 1302 pm"             },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %I%M %P",        "EDT",  "2016-07-04 0102 PM"             },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %I%M %p",        "EDT",  "2016-07-04 0102 pm"             },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H%M%S",         "EDT",  "2016-07-04 130242"              },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H%M%S %P",      "EDT",  "2016-07-04 130242 PM"           },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H%M%S %p",      "EDT",  "2016-07-04 130242 pm"           },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %I%M%S %P",      "EDT",  "2016-07-04 010242 PM"           },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %I%M%S %p",      "EDT",  "2016-07-04 010242 pm"           },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H%M%S %z",      "EDT",  "2016-07-04 130242 -0400"        },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H:%M",          "EDT",  "2016-07-04 13:02"               },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H:%M %P",       "EDT",  "2016-07-04 13:02 PM"            },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H:%M %p",       "EDT",  "2016-07-04 13:02 pm"            },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %I:%M %P",       "EDT",  "2016-07-04 01:02 PM"            },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %I:%M %p",       "EDT",  "2016-07-04 01:02 pm"            },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H:%M:%S",       "EDT",  "2016-07-04 13:02:42"            },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H:%M:%S %P",    "EDT",  "2016-07-04 13:02:42 PM"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H:%M:%S %p",    "EDT",  "2016-07-04 13:02:42 pm"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %I:%M:%S %P",    "EDT",  "2016-07-04 01:02:42 PM"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %I:%M:%S %p",    "EDT",  "2016-07-04 01:02:42 pm"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H:%M:%S %z",    "EDT",  "2016-07-04 13:02:42 -0400"      },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H-%M",          "EDT",  "2016-07-04 13-02"               },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H-%M %P",       "EDT",  "2016-07-04 13-02 PM"            },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %H-%M %p",       "EDT",  "2016-07-04 13-02 pm"            },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %I-%M %P",       "EDT",  "2016-07-04 01-02 PM"            },
	{ 1467651720, "EST5EDT", "%Y-%m-%d %I-%M %p",       "EDT",  "2016-07-04 01-02 pm"            },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H-%M-%S",       "EDT",  "2016-07-04 13-02-42"            },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H-%M-%S %P",    "EDT",  "2016-07-04 13-02-42 PM"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H-%M-%S %p",    "EDT",  "2016-07-04 13-02-42 pm"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %I-%M-%S %P",    "EDT",  "2016-07-04 01-02-42 PM"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %I-%M-%S %p",    "EDT",  "2016-07-04 01-02-42 pm"         },
	{ 1467651762, "EST5EDT", "%Y-%m-%d %H-%M-%S %z",    "EDT",  "2016-07-04 13-02-42 -0400"      },

	{ 1467651600, "EST5EDT", "%Y-%m-%d T %H",           "EDT",  "2016-07-04 T 13"                }, 
	{ 1467651600, "EST5EDT", "%Y-%m-%d T %H %P",        "EDT",  "2016-07-04 T 13 PM"             }, 
	{ 1467651600, "EST5EDT", "%Y-%m-%d T %H %p",        "EDT",  "2016-07-04 T 13 pm"             }, 
	{ 1467651600, "EST5EDT", "%Y-%m-%d T %I %P",        "EDT",  "2016-07-04 T 01 PM"             }, 
	{ 1467651600, "EST5EDT", "%Y-%m-%d T %I %p",        "EDT",  "2016-07-04 T 01 pm"             }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H%M",         "EDT",  "2016-07-04 T 1302"              }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H%M %P",      "EDT",  "2016-07-04 T 1302 PM"           }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H%M %p",      "EDT",  "2016-07-04 T 1302 pm"           }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %I%M %P",      "EDT",  "2016-07-04 T 0102 PM"           }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %I%M %p",      "EDT",  "2016-07-04 T 0102 pm"           }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H%M%S",       "EDT",  "2016-07-04 T 130242"            }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H%M%S %P",    "EDT",  "2016-07-04 T 130242 PM"         }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H%M%S %p",    "EDT",  "2016-07-04 T 130242 pm"         }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %I%M%S %P",    "EDT",  "2016-07-04 T 010242 PM"         }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %I%M%S %p",    "EDT",  "2016-07-04 T 010242 pm"         }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H%M%S %z",    "EDT",  "2016-07-04 T 130242 -0400"      }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H:%M",        "EDT",  "2016-07-04 T 13:02"             }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H:%M %P",     "EDT",  "2016-07-04 T 13:02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H:%M %p",     "EDT",  "2016-07-04 T 13:02 pm"          }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %I:%M %P",     "EDT",  "2016-07-04 T 01:02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %I:%M %p",     "EDT",  "2016-07-04 T 01:02 pm"          }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H:%M:%S",     "EDT",  "2016-07-04 T 13:02:42"          }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H:%M:%S %P",  "EDT",  "2016-07-04 T 13:02:42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H:%M:%S %p",  "EDT",  "2016-07-04 T 13:02:42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %I:%M:%S %P",  "EDT",  "2016-07-04 T 01:02:42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %I:%M:%S %p",  "EDT",  "2016-07-04 T 01:02:42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H:%M:%S %z",  "EDT",  "2016-07-04 T 13:02:42 -0400"    }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H-%M",        "EDT",  "2016-07-04 T 13-02"             }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H-%M %P",     "EDT",  "2016-07-04 T 13-02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %H-%M %p",     "EDT",  "2016-07-04 T 13-02 pm"          }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %I-%M %P",     "EDT",  "2016-07-04 T 01-02 PM"          }, 
	{ 1467651720, "EST5EDT", "%Y-%m-%d T %I-%M %p",     "EDT",  "2016-07-04 T 01-02 pm"          }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H-%M-%S",     "EDT",  "2016-07-04 T 13-02-42"          }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H-%M-%S %P",  "EDT",  "2016-07-04 T 13-02-42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H-%M-%S %p",  "EDT",  "2016-07-04 T 13-02-42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %I-%M-%S %P",  "EDT",  "2016-07-04 T 01-02-42 PM"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %I-%M-%S %p",  "EDT",  "2016-07-04 T 01-02-42 pm"       }, 
	{ 1467651762, "EST5EDT", "%Y-%m-%d T %H-%M-%S %z",  "EDT",  "2016-07-04 T 13-02-42 -0400"    }, 
                 
	{ 1467604800, "EST5EDT", "%m/%d/%Y",                "EDT",  "07/04/2016"                     },
	{ 1467604800, "EST5EDT", "%m-%d-%Y",                "EDT",  "07-04-2016"                     },
	{ 1467604800, "EST5EDT", "%m-%d-%y",                "EDT",  "07-04-16"                       },
	{ 1467604800, "EST5EDT", "%m/%d/%y",                "EDT",  "07/04/16"                       },
	{ 1467604800, "EST5EDT", "%m%d%Y",                  "EDT",  "07042016"                       },
	{ 1467604800, "EST5EDT", "%m%d%y",                  "EDT",  "070416"                         },
	{ 1467604800, "EST5EDT", "%Y/%m/%d",                "EDT",  "2016/07/04"                     },
	{ 1467604800, "EST5EDT", "%Y-%m-%d",                "EDT",  "2016-07-04"                     },

	{ 0, NULL, NULL, NULL, NULL }
};

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

START_TEST(check_time_fmt)
{
	M_time_tzs_t      *tzs;
	const M_time_tz_t *tz;
	char              *out;
	M_time_localtm_t   tm;
	M_time_t           ts;
	size_t             i;

	tzs = M_time_tzs_create();
	M_time_tzs_add_posix_str(tzs, EST5EDT);
	M_time_tzs_add_posix_str(tzs, PST8PDT);

	for (i=0; check_fmts[i].ts!=0; i++) {
		M_mem_set(&tm, 0, sizeof(tm));
		tz = M_time_tzs_get_tz(tzs, check_fmts[i].zone);

		M_time_tolocal(check_fmts[i].ts, &tm, tz);

		out = M_time_to_str(check_fmts[i].tostr_fmt, &tm);
		ck_assert_msg(M_str_eq(check_fmts[i].s, out), "%zu: Expected string '%s' does not match returned '%s'", i, check_fmts[i].s, out);

		ts = M_time_from_str(out, tz, M_FALSE);
		ck_assert_msg(check_fmts[i].ts == ts, "%zu: Expected ts %lld does not match returned %lld", i, (M_int64)check_fmts[i].ts, ts);
		M_free(out);

		out = M_time_to_str("%Z", &tm);
		ck_assert_msg(M_str_eq(check_fmts[i].abbr, out), "%zu: Expected abbr '%s' does not match returned '%s'", i, check_fmts[i].abbr, out);
		ck_assert_msg(M_str_eq(out, tm.abbr), "%zu: Expected abbr '%s' does not match tm.abbr '%s'", i, out, tm.abbr);
		M_free(out);
	}

	M_time_tzs_destroy(tzs);
}
END_TEST

static struct {
	M_time_t    ts;
	const char *zone;
	const char *tostr_fmt;
	const char *abbr;
	const char *s;
	M_bool      fail;
} check_fmts_custom[] = {
	{ 1361577553, "EST5EDT", "%Y/%m/%d %I:%M:%S%z %p",  "EST", "2013/02/22 06:59:13-0500 pm", M_FALSE },
	{ 1361534353, "EST5EDT", "%Y/%m/%d %I:%M:%S%z %p",  "EST", "2013/02/22 06:59:13-0500 am", M_FALSE },
	{ 1361534353, "EST5EDT", "%Y/%m/%d %H:%M:%S%z %p",  "EST", "2013/02/22 06:59:13-0500 am", M_FALSE },
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S%z %p",  "EST", "2013/02/22 18:59:13-0500 pm", M_FALSE },
	{ 1361577553, "EST5EDT", "%Y/%m/%d %H:%M:%S%z %p",  "EST", "2013/02/22 18:59:13-0500 am", M_TRUE  },
	{ 0, NULL, NULL, NULL, NULL, M_FALSE }
};


START_TEST(check_time_fmt_custom)
{
	M_time_tzs_t      *tzs;
	const M_time_tz_t *tz;
	char              *out;
	M_time_localtm_t   tm;
	M_time_t           ts;
	size_t             i;

	tzs = M_time_tzs_create();
	M_time_tzs_add_posix_str(tzs, EST5EDT);
	M_time_tzs_add_posix_str(tzs, PST8PDT);

	for (i=0; check_fmts_custom[i].ts!=0; i++) {
		M_mem_set(&tm, 0, sizeof(tm));
		tz = M_time_tzs_get_tz(tzs, check_fmts[i].zone);

		/* Don't free out. I points to a place in the input string. */
		out = M_time_parsefmt(check_fmts_custom[i].s, check_fmts_custom[i].tostr_fmt, &tm);

		if (check_fmts_custom[i].fail) {
			ck_assert_msg(out == NULL, "%zu: Parse succeeded when it should have failed", i);
			continue;
		} else {
			ck_assert_msg(out != NULL && *out == '\0', "%zu: Did not fully parse data: %s", i);
		}
		ts = M_time_fromlocal(&tm, tz);
		ck_assert_msg(check_fmts_custom[i].ts == ts, "%zu: Expected ts %lld does not match returned %lld", i, check_fmts_custom[i].ts, ts);

		out = M_time_to_str(check_fmts_custom[i].tostr_fmt, &tm);
		ck_assert_msg(M_str_eq(check_fmts_custom[i].s, out), "%zu: Expected string '%s' does not match returned '%s'", i, check_fmts_custom[i].s, out);
		M_free(out);
	}

	M_time_tzs_destroy(tzs);
}
END_TEST

static struct {
	M_time_t    ts;
	const char *s;
} check_fmts_utc[] = {
	{ 1361577553, "2013/02/22 23:59:13Z"  },
	{ 1361577553, "2013/02/22 T 23:59:13Z"  },
	{ 1377212353, "2013/08/22 22:59:13 Z" },
	{ 1377212353, "2013/08/22T22:59:13 Z" },
	{ 1377212353, "2013/08/22T22:59:13 UTC" },
	{ 1377212353, "2013/08/22T22:59:13 GMT" },
	{ 0, NULL}
};

START_TEST(check_time_fmt_utc)
{
	M_time_t ts;
	size_t   i;

	for (i=0; check_fmts_utc[i].ts!=0; i++) {
		ts = M_time_from_str(check_fmts_utc[i].s, NULL, M_FALSE);
		ck_assert_msg(check_fmts_utc[i].ts == ts, "%zu: Expected ts %lld does not match returned %lld", i, (M_int64)check_fmts_utc[i].ts, ts);
	}
}
END_TEST

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

Suite *M_time_fmt_suite(void)
{
	Suite *suite;
	TCase *tc_time_fmt;
	TCase *tc_time_fmt_utc;
	TCase *tc_time_fmt_custom;

	suite = suite_create("time_fmt");

	tc_time_fmt = tcase_create("time_fmt");
	tcase_add_unchecked_fixture(tc_time_fmt, NULL, NULL);
	tcase_add_test(tc_time_fmt, check_time_fmt);
	suite_add_tcase(suite, tc_time_fmt);

	tc_time_fmt_custom = tcase_create("time_fmt_custom");
	tcase_add_unchecked_fixture(tc_time_fmt_custom, NULL, NULL);
	tcase_add_test(tc_time_fmt_custom, check_time_fmt_custom);
	suite_add_tcase(suite, tc_time_fmt_custom);

	tc_time_fmt_utc = tcase_create("time_fmt_utc");
	tcase_add_unchecked_fixture(tc_time_fmt_utc, NULL, NULL);
	tcase_add_test(tc_time_fmt_utc, check_time_fmt_utc);
	suite_add_tcase(suite, tc_time_fmt_utc);

	return suite;
}

int main(void)
{
	int nf;
	SRunner *sr = srunner_create(M_time_fmt_suite());
	srunner_set_log(sr, "check_time_fmt.log");

	srunner_run_all(sr, CK_NORMAL);
	nf = srunner_ntests_failed(sr);
	srunner_free(sr);

	return nf == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}
